package com.devdaily.meditation;

import java.awt.Color;
import java.awt.Font;
import javax.swing.*;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.factories.ButtonBarFactory;

/**
 * Copyright 2009 Alvin J. Alexander, devdaily.com.
 * 
 * This file is part of my "Meditation" application.
 *
 * The Meditation application is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * The Meditation application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Meditation application. If not, see <http://www.gnu.org/licenses/>.
 *
 */
public class MeditationPanel
extends JPanel
{
  private JLabel headerLabel = new JLabel();
  private JLabel helpTextLabel = new JLabel();
  private JLabel timeLabel = new JLabel();
  private JLabel timeCountdownLabel = new JLabel();
  
  private JTextField timeEntryField=new JTextField(20);
  private JButton startStopButton = new JButton("Start");
  private JButton testSoundButton = new JButton("Test Sound");

  public JTextField getTimeField()
  {
    return timeEntryField;
  }
  
  public JButton getStartStopButton()
  {
    return startStopButton;
  }
  
  public JButton getTestSoundButton()
  {
    return testSoundButton;
  }
  
  public JLabel getTimeCountdownField()
  {
    return timeCountdownLabel;
  }
  
  public MeditationPanel()
  {
    try
    {
      headerLabel.setText("Meditation Timer");
      headerLabel.setFont(new Font("Tahoma", Font.BOLD, 14));
      helpTextLabel.setText("Set the desired meditation time, then click the Start button.");
      timeLabel.setText("Desired Time (minutes):");
      timeEntryField.setText("");
      timeCountdownLabel.setForeground(Color.GRAY);
      buildMeditationPanel();
    }
    catch(Exception exception)
    {
      // do nothing right now
    }
  }

  private void buildMeditationPanel()
  {
    FormLayout layout = new FormLayout("pref, 2dlu, pref:grow",                                  // columns
                        "p,3dlu, p,9dlu, p,3dlu, p,9dlu, p,3dlu, fill:pref:grow,3dlu, p,9dlu");  // rows
    PanelBuilder builder = new PanelBuilder(layout, this);
    builder.setDefaultDialogBorder();

    CellConstraints cc = new CellConstraints();

    // row 1
    builder.add(headerLabel, cc.xyw(1,1,3));

    // row 3
    builder.add(helpTextLabel, cc.xyw(1,3,3,"left,top"));

    // row 5
    builder.add(timeLabel, cc.xy(1,5));
    builder.add(timeEntryField, cc.xy(3,5));
    timeLabel.setLabelFor(timeEntryField);

    // row 7 (buttons)
    JButton[] buttons = {testSoundButton, startStopButton};
    JPanel buttonPanel = ButtonBarFactory.buildLeftAlignedBar(buttons);
    builder.add(buttonPanel, cc.xy(3,7));
    
    // row 9 (countdown label)
    builder.add(timeCountdownLabel, cc.xy(3,9));
  }

}








